import type * as cxapi from '@aws-cdk/cx-api';
import { CloudAssembly, StackCollection } from '../../aws-cdk';
import type { StackSelector } from '../stack-selector';
import type { ICloudAssemblySource } from '../types';
/**
 * A single Cloud Assembly wrapped to provide additional stack operations.
 */
export declare class StackAssembly extends CloudAssembly implements ICloudAssemblySource {
    produce(): Promise<cxapi.CloudAssembly>;
    /**
     * Improved stack selection interface with a single selector
     * @throws when the assembly does not contain any stacks, unless `selector.failOnEmpty` is `false`
     * @throws when individual selection strategies are not satisfied
     */
    selectStacksV2(selector: StackSelector): Promise<StackCollection>;
    /**
     * Select all stacks.
     *
     * This method never throws and can safely be used as a basis for other calculations.
     *
     * @returns a `StackCollection` of all stacks
     */
    selectAllStacks(): StackCollection;
    /**
     * Select all stacks that have the validateOnSynth flag et.
     *
     * @returns a `StackCollection` of all stacks that needs to be validated
     */
    selectStacksForValidation(): StackCollection;
}
